import datetime
import json
import re

from .common import InfoExtractor
from ..utils import (
    remove_start,
)


class BlinkxIE(InfoExtractor):
    _VALID_URL = r'^(?:https?://(?:www\.)blinkx\.com/ce/|blinkx:)(?P<id>[^?]+)'
    _IE_NAME = u'blinkx'

    _TEST = {
        u'url': u'http://www.blinkx.com/ce/8aQUy7GVFYgFzpKhT0oqsilwOGFRVXk3R1ZGWWdGenBLaFQwb3FzaWx3OGFRVXk3R1ZGWWdGenB',
        u'file': u'8aQUy7GV.mp4',
        u'md5': u'2e9a07364af40163a908edbf10bb2492',
        u'info_dict': {
            u"title": u"Police Car Rolls Away",
            u"uploader": u"stupidvideos.com",
            u"upload_date": u"20131215",
            u"description": u"A police car gently rolls away from a fight. Maybe it felt weird being around a confrontation and just had to get out of there!",
            u"duration": 14.886,
            u"thumbnails": [{
                "width": 100,
                "height": 76,
                "url": "http://cdn.blinkx.com/stream/b/41/StupidVideos/20131215/1873969261/1873969261_tn_0.jpg",
            }],
        },
    }

    def _real_extract(self, url):
        m = re.match(self._VALID_URL, url)
        video_id = m.group('id')
        display_id = video_id[:8]

        api_url = (u'https://apib4.blinkx.com/api.php?action=play_video&' +
                   u'video=%s' % video_id)
        data_json = self._download_webpage(api_url, display_id)
        data = json.loads(data_json)['api']['results'][0]
        dt = datetime.datetime.fromtimestamp(data['pubdate_epoch'])
        upload_date = dt.strftime('%Y%m%d')

        duration = None
        thumbnails = []
        formats = []
        for m in data['media']:
            if m['type'] == 'jpg':
                thumbnails.append({
                    'url': m['link'],
                    'width': int(m['w']),
                    'height': int(m['h']),
                })
            elif m['type'] == 'original':
                duration = m['d']
            elif m['type'] in ('flv', 'mp4'):
                vcodec = remove_start(m['vcodec'], 'ff')
                acodec = remove_start(m['acodec'], 'ff')
                format_id = (u'%s-%sk-%s' %
                             (vcodec,
                              (int(m['vbr']) + int(m['abr'])) // 1000,
                              m['w']))
                formats.append({
                    'format_id': format_id,
                    'url': m['link'],
                    'vcodec': vcodec,
                    'acodec': acodec,
                    'abr': int(m['abr']) // 1000,
                    'vbr': int(m['vbr']) // 1000,
                    'width': int(m['w']),
                    'height': int(m['h']),
                })
        formats.sort(key=lambda f: (f['width'], f['vbr'], f['abr']))

        return {
            'id': display_id,
            'fullid': video_id,
            'title': data['title'],
            'formats': formats,
            'uploader': data['channel_name'],
            'upload_date': upload_date,
            'description': data.get('description'),
            'thumbnails': thumbnails,
            'duration': duration,
        }



import json
import re

from .common import InfoExtractor
from ..utils import (
    compat_str,
    compat_urlparse,
    ExtractorError,
)


class BandcampIE(InfoExtractor):
    IE_NAME = u'Bandcamp'
    _VALID_URL = r'http://.*?\.bandcamp\.com/track/(?P<title>.*)'
    _TESTS = [{
        u'url': u'http://youtube-dl.bandcamp.com/track/youtube-dl-test-song',
        u'file': u'1812978515.mp3',
        u'md5': u'cdeb30cdae1921719a3cbcab696ef53c',
        u'info_dict': {
            u"title": u"youtube-dl test song \"'/\\\u00e4\u21ad"
        },
        u'skip': u'There is a limit of 200 free downloads / month for the test song'
    }]

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        title = mobj.group('title')
        webpage = self._download_webpage(url, title)
        # We get the link to the free download page
        m_download = re.search(r'freeDownloadPage: "(.*?)"', webpage)
        if m_download is None:
            m_trackinfo = re.search(r'trackinfo: (.+),\s*?\n', webpage)
        if m_trackinfo:
            json_code = m_trackinfo.group(1)
            data = json.loads(json_code)

            for d in data:
                formats = [{
                    'format_id': 'format_id',
                    'url': format_url,
                    'ext': format_id.partition('-')[0]
                } for format_id, format_url in sorted(d['file'].items())]
                return {
                    'id': compat_str(d['id']),
                    'title': d['title'],
                    'formats': formats,
                }
        else:
            raise ExtractorError(u'No free songs found')

        download_link = m_download.group(1)
        id = re.search(r'var TralbumData = {(.*?)id: (?P<id>\d*?)$', 
                       webpage, re.MULTILINE|re.DOTALL).group('id')

        download_webpage = self._download_webpage(download_link, id,
                                                  'Downloading free downloads page')
        # We get the dictionary of the track from some javascrip code
        info = re.search(r'items: (.*?),$',
                         download_webpage, re.MULTILINE).group(1)
        info = json.loads(info)[0]
        # We pick mp3-320 for now, until format selection can be easily implemented.
        mp3_info = info[u'downloads'][u'mp3-320']
        # If we try to use this url it says the link has expired
        initial_url = mp3_info[u'url']
        re_url = r'(?P<server>http://(.*?)\.bandcamp\.com)/download/track\?enc=mp3-320&fsig=(?P<fsig>.*?)&id=(?P<id>.*?)&ts=(?P<ts>.*)$'
        m_url = re.match(re_url, initial_url)
        #We build the url we will use to get the final track url
        # This url is build in Bandcamp in the script download_bunde_*.js
        request_url = '%s/statdownload/track?enc=mp3-320&fsig=%s&id=%s&ts=%s&.rand=665028774616&.vrs=1' % (m_url.group('server'), m_url.group('fsig'), id, m_url.group('ts'))
        final_url_webpage = self._download_webpage(request_url, id, 'Requesting download url')
        # If we could correctly generate the .rand field the url would be
        #in the "download_url" key
        final_url = re.search(r'"retry_url":"(.*?)"', final_url_webpage).group(1)

        track_info = {'id':id,
                      'title' : info[u'title'],
                      'ext' :   'mp3',
                      'url' :   final_url,
                      'thumbnail' : info[u'thumb_url'],
                      'uploader' :  info[u'artist']
                      }

        return [track_info]


class BandcampAlbumIE(InfoExtractor):
    IE_NAME = u'Bandcamp:album'
    _VALID_URL = r'http://.*?\.bandcamp\.com/album/(?P<title>.*)'

    _TEST = {
        u'url': u'http://blazo.bandcamp.com/album/jazz-format-mixtape-vol-1',
        u'playlist': [
            {
                u'file': u'1353101989.mp3',
                u'md5': u'39bc1eded3476e927c724321ddf116cf',
                u'info_dict': {
                    u'title': u'Intro',
                }
            },
            {
                u'file': u'38097443.mp3',
                u'md5': u'1a2c32e2691474643e912cc6cd4bffaa',
                u'info_dict': {
                    u'title': u'Kero One - Keep It Alive (Blazo remix)',
                }
            },
        ],
        u'params': {
            u'playlistend': 2
        },
        u'skip': u'Bancamp imposes download limits. See test_playlists:test_bandcamp_album for the playlist test'
    }

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        title = mobj.group('title')
        webpage = self._download_webpage(url, title)
        tracks_paths = re.findall(r'<a href="(.*?)" itemprop="url">', webpage)
        if not tracks_paths:
            raise ExtractorError(u'The page doesn\'t contain any track')
        entries = [
            self.url_result(compat_urlparse.urljoin(url, t_path), ie=BandcampIE.ie_key())
            for t_path in tracks_paths]
        title = self._search_regex(r'album_title : "(.*?)"', webpage, u'title')
        return {
            '_type': 'playlist',
            'title': title,
            'entries': entries,
        }



# encoding: utf-8

import re
import json
import hashlib
import uuid

from .common import InfoExtractor
from ..utils import (
    compat_urllib_parse,
    compat_urllib_request,
    ExtractorError,
)


class SmotriIE(InfoExtractor):
    IE_DESC = u'Smotri.com'
    IE_NAME = u'smotri'
    _VALID_URL = r'^https?://(?:www\.)?(?P<url>smotri\.com/video/view/\?id=(?P<videoid>v(?P<realvideoid>[0-9]+)[a-z0-9]{4}))'

    _TESTS = [
        # real video id 2610366
        {
            u'url': u'http://smotri.com/video/view/?id=v261036632ab',
            u'file': u'v261036632ab.mp4',
            u'md5': u'2a7b08249e6f5636557579c368040eb9',
            u'info_dict': {
                u'title': u'катастрофа с камер видеонаблюдения',
                u'uploader': u'rbc2008',
                u'uploader_id': u'rbc08',
                u'upload_date': u'20131118',
                u'description': u'катастрофа с камер видеонаблюдения, видео катастрофа с камер видеонаблюдения',
                u'thumbnail': u'http://frame6.loadup.ru/8b/a9/2610366.3.3.jpg',
            },
        },
        # real video id 57591
        {
            u'url': u'http://smotri.com/video/view/?id=v57591cb20',
            u'file': u'v57591cb20.flv',
            u'md5': u'830266dfc21f077eac5afd1883091bcd',
            u'info_dict': {
                u'title': u'test',
                u'uploader': u'Support Photofile@photofile',
                u'uploader_id': u'support-photofile',
                u'upload_date': u'20070704',
                u'description': u'test, видео test',
                u'thumbnail': u'http://frame4.loadup.ru/03/ed/57591.2.3.jpg',
            },
        },
        # video-password
        {
            u'url': u'http://smotri.com/video/view/?id=v1390466a13c',
            u'file': u'v1390466a13c.mp4',
            u'md5': u'f6331cef33cad65a0815ee482a54440b',
            u'info_dict': {
                u'title': u'TOCCA_A_NOI_-_LE_COSE_NON_VANNO_CAMBIAMOLE_ORA-1',
                u'uploader': u'timoxa40',
                u'uploader_id': u'timoxa40',
                u'upload_date': u'20100404',
                u'thumbnail': u'http://frame7.loadup.ru/af/3f/1390466.3.3.jpg',
                u'description': u'TOCCA_A_NOI_-_LE_COSE_NON_VANNO_CAMBIAMOLE_ORA-1, видео TOCCA_A_NOI_-_LE_COSE_NON_VANNO_CAMBIAMOLE_ORA-1',
            },
            u'params': {
                u'videopassword': u'qwerty',
            },
        },
        # age limit + video-password
        {
            u'url': u'http://smotri.com/video/view/?id=v15408898bcf',
            u'file': u'v15408898bcf.flv',
            u'md5': u'91e909c9f0521adf5ee86fbe073aad70',
            u'info_dict': {
                u'title': u'этот ролик не покажут по ТВ',
                u'uploader': u'zzxxx',
                u'uploader_id': u'ueggb',
                u'upload_date': u'20101001',
                u'thumbnail': u'http://frame3.loadup.ru/75/75/1540889.1.3.jpg',
                u'age_limit': 18,
                u'description': u'этот ролик не покажут по ТВ, видео этот ролик не покажут по ТВ',
            },
            u'params': {
                u'videopassword': u'333'
            }
        }
    ]
    
    _SUCCESS = 0
    _PASSWORD_NOT_VERIFIED = 1
    _PASSWORD_DETECTED = 2
    _VIDEO_NOT_FOUND = 3

    def _search_meta(self, name, html, display_name=None):
        if display_name is None:
            display_name = name
        return self._html_search_regex(
            r'<meta itemprop="%s" content="([^"]+)" />' % re.escape(name),
            html, display_name, fatal=False)
        return self._html_search_meta(name, html, display_name)

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        video_id = mobj.group('videoid')
        real_video_id = mobj.group('realvideoid')

        # Download video JSON data
        video_json_url = 'http://smotri.com/vt.php?id=%s' % real_video_id
        video_json_page = self._download_webpage(video_json_url, video_id, u'Downloading video JSON')
        video_json = json.loads(video_json_page)
        
        status = video_json['status']
        if status == self._VIDEO_NOT_FOUND:
            raise ExtractorError(u'Video %s does not exist' % video_id, expected=True)
        elif status == self._PASSWORD_DETECTED:  # The video is protected by a password, retry with
                                                # video-password set
            video_password = self._downloader.params.get('videopassword', None)
            if not video_password:
                raise ExtractorError(u'This video is protected by a password, use the --video-password option', expected=True)
            video_json_url += '&md5pass=%s' % hashlib.md5(video_password.encode('utf-8')).hexdigest()
            video_json_page = self._download_webpage(video_json_url, video_id, u'Downloading video JSON (video-password set)')
            video_json = json.loads(video_json_page)
            status = video_json['status']
            if status == self._PASSWORD_NOT_VERIFIED:
                raise ExtractorError(u'Video password is invalid', expected=True)
        
        if status != self._SUCCESS:
            raise ExtractorError(u'Unexpected status value %s' % status)
        
        # Extract the URL of the video
        video_url = video_json['file_data']
        
        # Video JSON does not provide enough meta data
        # We will extract some from the video web page instead
        video_page_url = 'http://' + mobj.group('url')
        video_page = self._download_webpage(video_page_url, video_id, u'Downloading video page')
        
        # Adult content
        if re.search(u'EroConfirmText">', video_page) is not None:
            self.report_age_confirmation()
            confirm_string = self._html_search_regex(
                r'<a href="/video/view/\?id=%s&confirm=([^"]+)" title="[^"]+">' % video_id,
                video_page, u'confirm string')
            confirm_url = video_page_url + '&confirm=%s' % confirm_string
            video_page = self._download_webpage(confirm_url, video_id, u'Downloading video page (age confirmed)')
            adult_content = True
        else:
            adult_content = False
        
        # Extract the rest of meta data
        video_title = self._search_meta(u'name', video_page, u'title')
        if not video_title:
            video_title = video_url.rsplit('/', 1)[-1]

        video_description = self._search_meta(u'description', video_page)
        END_TEXT = u' на сайте Smotri.com'
        if video_description.endswith(END_TEXT):
            video_description = video_description[:-len(END_TEXT)]
        START_TEXT = u'Смотреть онлайн ролик '
        if video_description.startswith(START_TEXT):
            video_description = video_description[len(START_TEXT):]
        video_thumbnail = self._search_meta(u'thumbnail', video_page)

        upload_date_str = self._search_meta(u'uploadDate', video_page, u'upload date')
        upload_date_m = re.search(r'(?P<year>\d{4})\.(?P<month>\d{2})\.(?P<day>\d{2})T', upload_date_str)
        video_upload_date = (
            (
                upload_date_m.group('year') +
                upload_date_m.group('month') +
                upload_date_m.group('day')
            )
            if upload_date_m else None
        )
        
        duration_str = self._search_meta(u'duration', video_page)
        duration_m = re.search(r'T(?P<hours>[0-9]{2})H(?P<minutes>[0-9]{2})M(?P<seconds>[0-9]{2})S', duration_str)
        video_duration = (
            (
                (int(duration_m.group('hours')) * 60 * 60) +
                (int(duration_m.group('minutes')) * 60) +
                int(duration_m.group('seconds'))
            )
            if duration_m else None
        )
        
        video_uploader = self._html_search_regex(
            u'<div class="DescrUser"><div>Автор.*?onmouseover="popup_user_info[^"]+">(.*?)</a>',
            video_page, u'uploader', fatal=False, flags=re.MULTILINE|re.DOTALL)
        
        video_uploader_id = self._html_search_regex(
            u'<div class="DescrUser"><div>Автор.*?onmouseover="popup_user_info\\(.*?\'([^\']+)\'\\);">',
            video_page, u'uploader id', fatal=False, flags=re.MULTILINE|re.DOTALL)
        
        video_view_count = self._html_search_regex(
            u'Общее количество просмотров.*?<span class="Number">(\\d+)</span>',
            video_page, u'view count', fatal=False, flags=re.MULTILINE|re.DOTALL)
                
        return {
            'id': video_id,
            'url': video_url,
            'title': video_title,
            'thumbnail': video_thumbnail,
            'description': video_description,
            'uploader': video_uploader,
            'upload_date': video_upload_date,
            'uploader_id': video_uploader_id,
            'video_duration': video_duration,
            'view_count': video_view_count,
            'age_limit': 18 if adult_content else 0,
            'video_page_url': video_page_url
        }


class SmotriCommunityIE(InfoExtractor):
    IE_DESC = u'Smotri.com community videos'
    IE_NAME = u'smotri:community'
    _VALID_URL = r'^https?://(?:www\.)?smotri\.com/community/video/(?P<communityid>[0-9A-Za-z_\'-]+)'
    
    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        community_id = mobj.group('communityid')

        url = 'http://smotri.com/export/rss/video/by/community/-/%s/video.xml' % community_id
        rss = self._download_xml(url, community_id, u'Downloading community RSS')

        entries = [self.url_result(video_url.text, 'Smotri')
                   for video_url in rss.findall('./channel/item/link')]

        description_text = rss.find('./channel/description').text
        community_title = self._html_search_regex(
            u'^Видео сообщества "([^"]+)"$', description_text, u'community title')

        return self.playlist_result(entries, community_id, community_title)


class SmotriUserIE(InfoExtractor):
    IE_DESC = u'Smotri.com user videos'
    IE_NAME = u'smotri:user'
    _VALID_URL = r'^https?://(?:www\.)?smotri\.com/user/(?P<userid>[0-9A-Za-z_\'-]+)'

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        user_id = mobj.group('userid')

        url = 'http://smotri.com/export/rss/user/video/-/%s/video.xml' % user_id
        rss = self._download_xml(url, user_id, u'Downloading user RSS')

        entries = [self.url_result(video_url.text, 'Smotri')
                   for video_url in rss.findall('./channel/item/link')]

        description_text = rss.find('./channel/description').text
        user_nickname = self._html_search_regex(
            u'^Видео режиссера (.*)$', description_text,
            u'user nickname')

        return self.playlist_result(entries, user_id, user_nickname)


class SmotriBroadcastIE(InfoExtractor):
    IE_DESC = u'Smotri.com broadcasts'
    IE_NAME = u'smotri:broadcast'
    _VALID_URL = r'^https?://(?:www\.)?(?P<url>smotri\.com/live/(?P<broadcastid>[^/]+))/?.*'

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        broadcast_id = mobj.group('broadcastid')

        broadcast_url = 'http://' + mobj.group('url')
        broadcast_page = self._download_webpage(broadcast_url, broadcast_id, u'Downloading broadcast page')

        if re.search(u'>Режиссер с логином <br/>"%s"<br/> <span>не существует<' % broadcast_id, broadcast_page) is not None:
            raise ExtractorError(u'Broadcast %s does not exist' % broadcast_id, expected=True)

        # Adult content
        if re.search(u'EroConfirmText">', broadcast_page) is not None:

            (username, password) = self._get_login_info()
            if username is None:
                raise ExtractorError(u'Erotic broadcasts allowed only for registered users, '
                    u'use --username and --password options to provide account credentials.', expected=True)

            # Log in
            login_form_strs = {
                u'login-hint53': '1',
                u'confirm_erotic': '1',
                u'login': username,
                u'password': password,
            }
            # Convert to UTF-8 *before* urlencode because Python 2.x's urlencode
            # chokes on unicode
            login_form = dict((k.encode('utf-8'), v.encode('utf-8')) for k,v in login_form_strs.items())
            login_data = compat_urllib_parse.urlencode(login_form).encode('utf-8')
            login_url = broadcast_url + '/?no_redirect=1'
            request = compat_urllib_request.Request(login_url, login_data)
            request.add_header('Content-Type', 'application/x-www-form-urlencoded')
            broadcast_page = self._download_webpage(
                request, broadcast_id, note=u'Logging in and confirming age')

            if re.search(u'>Неверный логин или пароль<', broadcast_page) is not None:
                raise ExtractorError(u'Unable to log in: bad username or password', expected=True)

            adult_content = True
        else:
            adult_content = False

        ticket = self._html_search_regex(
            u'window\.broadcast_control\.addFlashVar\\(\'file\', \'([^\']+)\'\\);',
            broadcast_page, u'broadcast ticket')

        url = 'http://smotri.com/broadcast/view/url/?ticket=%s' % ticket

        broadcast_password = self._downloader.params.get('videopassword', None)
        if broadcast_password:
            url += '&pass=%s' % hashlib.md5(broadcast_password.encode('utf-8')).hexdigest()

        broadcast_json_page = self._download_webpage(url, broadcast_id, u'Downloading broadcast JSON')

        try:
            broadcast_json = json.loads(broadcast_json_page)

            protected_broadcast = broadcast_json['_pass_protected'] == 1
            if protected_broadcast and not broadcast_password:
                raise ExtractorError(u'This broadcast is protected by a password, use the --video-password option', expected=True)

            broadcast_offline = broadcast_json['is_play'] == 0
            if broadcast_offline:
                raise ExtractorError(u'Broadcast %s is offline' % broadcast_id, expected=True)

            rtmp_url = broadcast_json['_server']
            if not rtmp_url.startswith('rtmp://'):
                raise ExtractorError(u'Unexpected broadcast rtmp URL')

            broadcast_playpath = broadcast_json['_streamName']
            broadcast_thumbnail = broadcast_json['_imgURL']
            broadcast_title = broadcast_json['title']
            broadcast_description = broadcast_json['description']
            broadcaster_nick = broadcast_json['nick']
            broadcaster_login = broadcast_json['login']
            rtmp_conn = 'S:%s' % uuid.uuid4().hex
        except KeyError:
            if protected_broadcast:
                raise ExtractorError(u'Bad broadcast password', expected=True)
            raise ExtractorError(u'Unexpected broadcast JSON')

        return {
            'id': broadcast_id,
            'url': rtmp_url,
            'title': broadcast_title,
            'thumbnail': broadcast_thumbnail,
            'description': broadcast_description,
            'uploader': broadcaster_nick,
            'uploader_id': broadcaster_login,
            'age_limit': 18 if adult_content else 0,
            'ext': 'flv',
            'play_path': broadcast_playpath,
            'rtmp_live': True,
            'rtmp_conn': rtmp_conn
        }



import re
import json

from .common import InfoExtractor
from ..utils import (
    compat_urlparse,
    get_element_by_attribute,
)


class ImdbIE(InfoExtractor):
    IE_NAME = u'imdb'
    IE_DESC = u'Internet Movie Database trailers'
    _VALID_URL = r'http://www\.imdb\.com/video/imdb/vi(?P<id>\d+)'

    _TEST = {
        u'url': u'http://www.imdb.com/video/imdb/vi2524815897',
        u'md5': u'9f34fa777ade3a6e57a054fdbcb3a068',
        u'info_dict': {
            u'id': u'2524815897',
            u'ext': u'mp4',
            u'title': u'Ice Age: Continental Drift Trailer (No. 2) - IMDb',
            u'description': u'md5:9061c2219254e5d14e03c25c98e96a81',
        }
    }

    def _real_extract(self, url):
        mobj = re.match(self._VALID_URL, url)
        video_id = mobj.group('id')
        webpage = self._download_webpage(url,video_id)
        descr = get_element_by_attribute('itemprop', 'description', webpage)
        available_formats = re.findall(
            r'case \'(?P<f_id>.*?)\' :$\s+url = \'(?P<path>.*?)\'', webpage,
            flags=re.MULTILINE)
        formats = []
        for f_id, f_path in available_formats:
            f_path = f_path.strip()
            format_page = self._download_webpage(
                compat_urlparse.urljoin(url, f_path),
                u'Downloading info for %s format' % f_id)
            json_data = self._search_regex(
                r'<script[^>]+class="imdb-player-data"[^>]*?>(.*?)</script>',
                format_page, u'json data', flags=re.DOTALL)
            info = json.loads(json_data)
            format_info = info['videoPlayerObject']['video']
            formats.append({
                'format_id': f_id,
                'url': format_info['url'],
            })

        return {
            'id': video_id,
            'title': self._og_search_title(webpage),
            'formats': formats,
            'description': descr,
            'thumbnail': format_info['slate'],
        }
